# *************************************************************** #
#               Copyright © Hero Imaging AB 2022. 				  #
#  					  All Rights Reserved.						  #
# *************************************************************** #

from __future__ import annotations
from typing import Callable, Any, Optional, Union
import numpy as np
from heropytools.Serialization.Attributes import Attributes
from heropytools.Serialization.CustomSerializer import CustomSerializer

from .CustomSerializers.TensorSerializer import TensorSerializer
from .HeroData import HeroData
from .HeroDataType import HeroDataType


class HeroArray(HeroData):

    def __init__(self, value: Union[np.ndarray, bool, int, float, complex]):
        
        HeroData.__init__(self)

        if isinstance(value, np.ndarray):
            self._value = value
        else:
            self._value = np.array(value)

    @staticmethod
    def create(obj):
        return HeroArray(obj)

    @staticmethod
    def create_from_dict(data: dict):
        return data['_value']

    def __eq__(self, other: HeroArray):
        if not super(HeroArray, self).__eq__(other):
            return False
        return (self._value == other._value).all()

    @property
    def is_boolean(self):
        return self._value.dtype == np.bool

    @is_boolean.setter
    def is_boolean(self, value):
        raise AttributeError("Attribute 'is_boolean' is not writable.")

    @is_boolean.deleter
    def is_boolean(self):
        raise AttributeError("Attribute 'is_boolean' is not deletable.")

    # --- Autogenerated --- #

    # Name of the type.
    _type_str = "HeroArray"

    # Datatype attributes.
    _attributes = Attributes("HA", version=0, member_count=1)

    # Serialization.
    def serialize(self, writer_fun: Callable[[Any, str, str, Optional[CustomSerializer]], None]):
        super(HeroArray, self).serialize(writer_fun)
        writer_fun(self._value, "Tensor", "V", TensorSerializer())

    # Deserialization.
    @staticmethod
    def deserialize(reader_fun: Callable[[str, Optional[CustomSerializer]], Any]):
        data = super(HeroArray, HeroArray).deserialize(reader_fun)
        data["_value"], _ = reader_fun("Tensor", TensorSerializer())
        return data
